/**********************************************************************
*
* IBM Confidential
*
* OCO Source Materials
*
* 5724-L66
*
* (c) Copyright IBM Corp.  2006 All Rights Reserved
*
* The source code for this program is not published or otherwise divested of its trade secrets,
* irrespective of what has been deposited with the U.S. Copyright Office.
* 
**/
/*
 * Created on Nov 21, 2005
 *
 */
package com.ibm.datatools.mdsi.modeltransform;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;

import com.ibm.db.models.naming.ClassType;
import com.ibm.db.models.naming.Glossary;
import com.ibm.db.models.naming.NamingModelFactory;
import com.ibm.db.models.naming.NamingModelPackage;
import com.ibm.db.models.naming.NamingStandard;
import com.ibm.db.models.naming.SynonymGroup;
import com.ibm.db.models.naming.Word;

/**
 * @author ellersr
 *
 */
public class NDMtoNDM extends Transformer 
{
	protected Set excludedWords;
	
	

	/**
	 * This constructor is used for the normal invocation of the NDMtoNDM API
	 * 
	 * @param options
	 */
	public NDMtoNDM(NDMtoNDMOptions options) {
		super(options);
	}
	protected NDMtoNDMOptions getNDMtoNDMOptions() {
		return (NDMtoNDMOptions)options;
	}
	protected Set getExcludedWords() {
		if (excludedWords==null) {
			excludedWords = new HashSet();
			Iterator e = getNDMtoNDMOptions().getExcludedTerms().iterator();
			while (e.hasNext()) {
				String term = (String) e.next();
				excludedWords.add(term.toUpperCase());
			}
			
		}
		return excludedWords;
	}
	/**
	 * This constructor is only used when the NDMtoNDM is being invoked from
	 * a headless workbench.
	 */
	public NDMtoNDM() {
		super();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IPlatformRunnable#run(java.lang.Object)
	 */
	public Object run(Object args) throws Exception {
		options = new NDMtoNDMOptions((String[])args);

		return run();
	}
	
	
	public static void main(String[] args) {
		NDMtoNDMOptions options = new NDMtoNDMOptions(args);
		NDMtoNDM transformer = new NDMtoNDM(options);
		
		transformer.run();
	}
	
	/**
	 * process()
	 * 
	 * Make a copy of the input NDM file and apply the following transformations:
	 * <ol>
	 * <li>Flatten out subGlossaries. The outermost glosssary is preserved. Nested glossaries
	 * are replaced by Words
	 * <li>If no namingStandard exists, create one
	 * <li>On option, generate new words from the components of multi word terms.
	 * I.e. look for words that are actually made up of multiple, blank separated
	 * words. When found, create new words from the component parts (assuming those
	 * words do not already exist.) Exclude any terms that were specified by 
	 * an options excludedTermList (e.g. "the", "a", "an", "of", etc...)
	 */
	protected void process(ResourceSet resourceSet, Resource outputResource) 
	{
	    Copier copier = new EcoreUtil.Copier();
	    Iterator r = resourceSet.getResources().iterator();
	    Collection newContents = new ArrayList();
	    while (r.hasNext()) {
	    	Resource resource = (Resource) r.next();
		    newContents.addAll(copier.copyAll(resource.getContents()));
	    }
	    copier.copyReferences();

	    
	    outputResource.getContents().addAll(newContents);
	    
	    List outputContents = outputResource.getContents();
	    
	    // Make sure a naming standard exists and it is in the first position in the resource
		NamingStandard rootNamingStandard = (NamingStandard)EcoreUtil.getObjectByType(outputContents, NamingModelPackage.eINSTANCE.getNamingStandard());
		NamingStandard generatedNamingStandard=null;
		if (rootNamingStandard==null) {
			rootNamingStandard = NamingModelFactory.eINSTANCE.createNamingStandard();
			outputResource.getContents().add(0,rootNamingStandard);
			String name=outputResource.getURI().lastSegment();
			String ext = outputResource.getURI().fileExtension();
			if (ext!=null && ext.length()>0) {
				name = name.substring(0,name.length()-ext.length()-1);
			}
			rootNamingStandard.setName(name);
			generatedNamingStandard=rootNamingStandard;
		} else {
			int pos = outputContents.indexOf(rootNamingStandard);
			if (pos!=0) {
				outputContents.remove(rootNamingStandard);
				outputContents.add(0,rootNamingStandard);
			}
		}
		
	    Glossary firstGlossary=null;
	    List otherGlossaries = new ArrayList();
		Iterator g = EcoreUtil.getObjectsByType(outputContents, NamingModelPackage.eINSTANCE.getGlossary()).iterator();
		// Iterate over the "root" glossaries. 
		// (Normally, we'd only expect there to be one outermost glossary, but the code will allow
		// for more than one.)
		while (g.hasNext()) {
			Glossary glossary = (Glossary)g.next();
			if (generatedNamingStandard!=null) {
				generatedNamingStandard.getGlossaries().add(glossary);
			}

			Iterator w = glossary.getWords().iterator();
			while (w.hasNext()) {
				Word word = (Word) w.next();
				addGlossaryToDescription(word, glossary);
				if (word.getDescription()==null) {
					word.setDescription(""); //$NON-NLS-1$
				}
				if (word.getAbbreviation()==null) {
					word.setAbbreviation(""); //$NON-NLS-1$
				}
				if (word.getAlternateAbbreviation()==null) {
					word.setAlternateAbbreviation(""); //$NON-NLS-1$
				}

			}
			
			Map wordMap= new WordMap(getNDMtoNDMOptions().isIgnoreCase());
			mapWords(wordMap, glossary.getWords());
			if (getNDMtoNDMOptions().isFlattenGlossaries()) {
				Collection newWords=flattenSubglossaries(wordMap, glossary.getSubGlossaries());
				glossary.getSubGlossaries().clear();
				mapWords(wordMap, newWords);
				glossary.getWords().addAll(newWords);
			}
			if (getNDMtoNDMOptions().isSplitTerms()) {
				glossary.getWords().addAll(splitTerms(wordMap,glossary.getWords()));
			}
			if (firstGlossary==null) {
				firstGlossary=glossary;
			} else {
				otherGlossaries.add(glossary);
			}
		}
		if (getNDMtoNDMOptions().isFlattenGlossaries()) {
			Iterator m = otherGlossaries.iterator();
			while (m.hasNext()) {
				Glossary mergedGlossary = (Glossary) m.next();
				outputContents.remove(mergedGlossary);
				rootNamingStandard.getGlossaries().remove(mergedGlossary);
				firstGlossary.getWords().add(glossaryToWord(mergedGlossary));
				firstGlossary.getWords().addAll(mergedGlossary.getWords());
			}
			Iterator w = EcoreUtil.getObjectsByType(outputContents, NamingModelPackage.eINSTANCE.getWord()).iterator();
			while (w.hasNext()) {
				Word word = (Word) w.next();
				firstGlossary.getWords().add(word);
			}
		}
		Iterator s = EcoreUtil.getObjectsByType(outputContents, NamingModelPackage.eINSTANCE.getSynonymGroup()).iterator();
		while (s.hasNext()) {
			SynonymGroup sGroup = (SynonymGroup) s.next();
			addSynonymsToDescription(sGroup);
		}
		
	}
	protected Word glossaryToWord(Glossary glossary) {
		Word newWord = NamingModelFactory.eINSTANCE.createWord();
		newWord.setName(glossary.getName());
		newWord.setDescription(""); //$NON-NLS-1$
		newWord.setAbbreviation(""); //$NON-NLS-1$
		newWord.setAlternateAbbreviation(""); //$NON-NLS-1$
		addCategoryFlagToDescription(newWord);
		ClassType  classification=ClassType.PRIME_LITERAL; 
		newWord.setClassification(classification);
		newWord.setModifier(false);
		return newWord;
	}
	protected void mapWords(Map wordMap, Collection wordList) {
		Iterator w = EcoreUtil.getObjectsByType(wordList, NamingModelPackage.eINSTANCE.getWord()).iterator();
		while (w.hasNext()) {
			Word word = (Word)w.next();
			wordMap.put(word.getName(),word);
		}
	}
	/**
	 * @param wordMap
	 * @param subglossaryList
	 * @return
	 */
	protected Collection flattenSubglossaries(Map wordMap, Collection subglossaryList) {
		Collection newWords = new ArrayList();
		Iterator g = EcoreUtil.getObjectsByType(subglossaryList, NamingModelPackage.eINSTANCE.getGlossary()).iterator();
		while (g.hasNext()) {
			Glossary glossary = (Glossary)g.next();
			String newName = glossary.getName();
			if (!wordMap.containsKey(newName)) {
				
				Word newWord = glossaryToWord(glossary);
				newWords.add(newWord);
				wordMap.put(newName, newWord);
			}

			Iterator w = glossary.getWords().iterator();
			while (w.hasNext()) {
				Word subWord = (Word)w.next();
				String subName = subWord.getName();
				if (!wordMap.containsKey(subName)) {
					newWords.add(subWord);
					wordMap.put(subName, subWord);
				}
				addGlossaryToDescription(subWord, glossary);
			}
			glossary.getWords().clear();
			newWords.addAll(flattenSubglossaries(wordMap, glossary.getSubGlossaries()));
		}
		return newWords;
	}
	private void addCategoryFlagToDescription(Word word) {
		if (getNDMtoNDMOptions().isUpdateDescription()) {
			String oldDescription = (word.getDescription()==null)?"":word.getDescription(); //$NON-NLS-1$
			if (oldDescription.length()>0) oldDescription = ModeltransformPlugin.getResourceString("_xform_description_separator")+//$NON-NLS-1$
			                                                " "+oldDescription; //$NON-NLS-1$
			word.setDescription(ModeltransformPlugin.getResourceString("_xform_description_open_paren")+ //$NON-NLS-1$
			                    ModeltransformPlugin.getResourceString("_xform_description_category")+ //$NON-NLS-1$
								ModeltransformPlugin.getResourceString("_xform_description_close_paren")+oldDescription); //$NON-NLS-1$
		}
	}
	private void addGlossaryToDescription(Word word, Glossary glossary) {
		if (getNDMtoNDMOptions().isUpdateDescription()) {
			if (glossary.getName()!=null && glossary.getName().length()>0) {
				String oldDescription = (word.getDescription()==null)?"":word.getDescription(); //$NON-NLS-1$
				if (!oldDescription.startsWith(ModeltransformPlugin.getResourceString("_xform_description_open_paren")+ //$NON-NLS-1$
						                       ModeltransformPlugin.getResourceString("_xform_description_from_category"))) { //$NON-NLS-1$
					if (oldDescription.length()>0) {
						oldDescription = ModeltransformPlugin.getResourceString("_xform_description_separator")+  //$NON-NLS-1$
                                         " "+oldDescription; //$NON-NLS-1$
					}
					word.setDescription(ModeltransformPlugin.getResourceString("_xform_description_open_paren")+
							ModeltransformPlugin.getResourceString("_xform_description_from_category")+ //$NON-NLS-1$
				            " \""+  //$NON-NLS-1$
							glossary.getName()+
							"\""+ //$NON-NLS-1$
							ModeltransformPlugin.getResourceString("_xform_description_close_paren")+ //$NON-NLS-1$
							oldDescription);
				}
			}
		}
	}
	private void addSynonymsToDescription(SynonymGroup sGroup) {
		if (getNDMtoNDMOptions().isUpdateDescription()) {
			Word preferred = sGroup.getPreferred();
			if (preferred!=null) {
				Iterator w = sGroup.getWords().iterator();
				while (w.hasNext()) {
					Word word = (Word) w.next();
					if (!word.equals(preferred)) {
						String oldDescription = (word.getDescription()==null)?"":word.getDescription(); //$NON-NLS-1$
						if (oldDescription.length()>0) oldDescription = ModeltransformPlugin.getResourceString("_xform_description_separator")+ //$NON-NLS-1$
											                            " "+oldDescription; //$NON-NLS-1$
						word.setDescription(ModeltransformPlugin.getResourceString("_xform_description_open_paren")+ //$NON-NLS-1$
								ModeltransformPlugin.getResourceString("_xform_description_synonym_for")+ //$NON-NLS-1$
								" \""+  //$NON-NLS-1$
					            preferred.getName()+
								"\""+  //$NON-NLS-1$
								ModeltransformPlugin.getResourceString("_xform_description_close_paren")+oldDescription);  //$NON-NLS-1$
					}
				}
			} else if (sGroup.getWords().size()>1){
				Iterator w = sGroup.getWords().iterator();
				int size = sGroup.getWords().size();
				Word[] synonyms = new Word[size];
				int i=0;
				while (w.hasNext()) {
					synonyms[i++] = (Word) w.next();
				}
				for (int j=0; j<size; j++)
				{
					String sList=""; //$NON-NLS-1$
					for (int k=0;k<size;k++) {
						if (k!=j) {
							if (sList.length()!=0) sList += ", "; //$NON-NLS-1$
							sList+="\""+  //$NON-NLS-1$
							       synonyms[k].getName()+
								   "\""; //$NON-NLS-1$
						}
					}
					Word word = synonyms[j];
					String oldDescription = (word.getDescription()==null)?"":word.getDescription(); //$NON-NLS-1$
					if (oldDescription.length()>0) {
						        oldDescription = ModeltransformPlugin.getResourceString("_xform_description_separator")+ //$NON-NLS-1$
								                 " "+oldDescription; //$NON-NLS-1$
					}
					word.setDescription(ModeltransformPlugin.getResourceString("_xform_description_open_paren")+ //$NON-NLS-1$
							ModeltransformPlugin.getResourceString("_xform_description_synonyms")+ //$NON-NLS-1$
							ModeltransformPlugin.getResourceString("_xform_description_separator")+ //$NON-NLS-1$
							" "+ //$NON-NLS-1$
							ModeltransformPlugin.getResourceString("_xform_description_open_brace")+ //$NON-NLS-1$
							sList+
							ModeltransformPlugin.getResourceString("_xform_description_close_brace")+ //$NON-NLS-1$
							ModeltransformPlugin.getResourceString("_xform_description_close_paren")+oldDescription); //$NON-NLS-1$
				}
			
			}
		}
	}
	protected Collection splitTerms(Map wordMap, Collection wordList) {
		Collection newWords = new ArrayList();
		Iterator w = wordList.iterator();
		while (w.hasNext()) {
			Word word = (Word) w.next();
			String name = word.getName();
			String[] subNames = name.split("[ \\t]+");  //$NON-NLS-1$
			List nameList= new ArrayList();
			for (int i=0; i<subNames.length; i++) {
				String subName = subNames[i];
				
				if (!getExcludedWords().contains(subName.toUpperCase()) && !wordMap.containsKey(subName)) {
					Word newWord = NamingModelFactory.eINSTANCE.createWord();
					newWord.setDescription(""); //$NON-NLS-1$
					newWord.setAbbreviation(""); //$NON-NLS-1$
					newWord.setAlternateAbbreviation(""); //$NON-NLS-1$
					
					newWords.add(newWord);
					wordMap.put(subName, newWord);
					newWord.setName(subName);
					newWord.setDescription(ModeltransformPlugin.getResourceString("_xform_description_split_from")+ //$NON-NLS-1$
				               " \""+ //$NON-NLS-1$
							   word.getName()+
							   "\""); //$NON-NLS-1$
				}
				nameList.add(subNames[i]);
			}
			System.out.println("splitTerms: "+name+" -> "+nameList); //$NON-NLS-1$ //$NON-NLS-2$
			
		}
		return newWords;
		
	}
	protected class WordMap extends HashMap {
		boolean ignoreCase=false;
		public WordMap(boolean ignoreCase) {
			this.ignoreCase=ignoreCase;
		}
		/* (non-Javadoc)
		 * @see java.util.Map#containsKey(java.lang.Object)
		 */
		public boolean containsKey(Object key) {
			if (ignoreCase) {
				return super.containsKey(((String)key).toUpperCase());
			} else {
				return super.containsKey(key);
			}
		}
		/* (non-Javadoc)
		 * @see java.util.Map#get(java.lang.Object)
		 */
		public Object get(Object key) {
			if (ignoreCase) {
				return super.get(((String)key).toUpperCase());
			} else {
				return super.get(key);
			}
		}
		/* (non-Javadoc)
		 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
		 */
		public Object put(Object key, Object value) {
			if (ignoreCase) {
				return super.put(((String)key).toUpperCase(),value);
			} else {
				return super.put(key, value);
			}
		}
		
	}
}
