/**********************************************************************
*
* IBM Confidential
*
* OCO Source Materials
*
* 5724-L66
*
* (c) Copyright IBM Corp.  2006 All Rights Reserved
*
* The source code for this program is not published or otherwise divested of its trade secrets,
* irrespective of what has been deposited with the U.S. Copyright Office.
* 
**/
/*
 * Created on Nov 21, 2005
 *
 */
package com.ibm.datatools.mdsi.modeltransform;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import com.ibm.datatools.internal.core.resource.DataModelResourceFactory;
import com.ibm.db.models.naming.NamingModelPackage;

/**
 * @author ellersr
 *
 */
public abstract class Transformer implements IPlatformRunnable 
{
	protected TransformerOptions options;
//	protected ResourceSet resourceSet;
//	protected Resource outputResource;
	
	

	/**
	 * This constructor is used for the normal invocation of the NDMtoNDM API
	 * 
	 * @param options
	 */
	public Transformer(TransformerOptions options) {
		this();
		this.options = options;
	}
	/**
	 * This constructor is only used when the NDMtoNDM is being invoked from
	 * a headless workbench.
	 */
	public Transformer() {
		super();
		// need to access the ecore package to avoid bootstrap problems...
		EPackage   ecorePackage = EcorePackage.eINSTANCE;
		EPackage namingModelPackage = NamingModelPackage.eINSTANCE;

		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				"tmp", new DataModelResourceFactory());
    
//		resourceSet = new ResourceSetImpl();
	}
	
	/**
	 * Used to run the transformer. Options should be specified by creating and instance of {@link NDMtoNDMOptions}
	 * and passing that instance to the constructor of NDMtoNDM.
	 * 
	 * @return
	 * @throws Exception
	 */
	public Object run() {

		ResourceSet resourceSet= new ResourceSetImpl();
		load(resourceSet);
		URI uri = URI.createURI(options.getOutputFileName());
		Resource outputResource = resourceSet.createResource(uri);
		process(resourceSet,outputResource);
		save(outputResource);
		return EXIT_OK;
	}
	
	
	protected void load(ResourceSet resourceSet) 
	{
		System.out.println("Transformer loading: " + options.getInputFileNames());
		List inFiles = options.getInputFileNames();
		
		Iterator i = inFiles.iterator();
		while (i.hasNext()) {
			String inFile = (String) i.next();
			URI uri = URI.createURI(inFile);
			Resource resource = resourceSet.createResource(uri);
			try {
				resource.load(null);
			}
			catch (IOException e) {
				System.out.println("Error while loading resource: "+ e);
			}
		}
		
	}
	protected void save(Resource resource) 
	{
		System.out.println("Transformer saving: " + options.getOutputFileName());
		
		try {
			resource.save(null);
		}
		catch (IOException e) {
			System.out.println("Error while saving resource: "+ e);
		}
	}
	
	/**
	 * process()
	 * 
	 */
	protected abstract void process(ResourceSet resourceSet, Resource resource) ;
}
