//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2002    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################


#ifndef MIRArrayInterface_h
#define MIRArrayInterface_h 1

#include "MIRContainer.h"

/*
 * Defines an interface for accessing objects stored in a container 
 * by their position. 
 */
class MIR_STD_API MIRArrayInterface: public virtual _MIRContainer {

  public:

    //## Position based methods for adding and removing elements

      virtual int addElement(const MIRContainerItem* ptr, MIRBoolean unique=true) = 0;
      //    Adds a new object to the container. If unique is true then
      //    the insertion will fail if an object with the same key
      //    is already in the container.
      //    Returns the position of the new element if the insertion 
      //    succeeds and -1 otherwise.

      virtual int containsElement(const MIRContainerItem* ptr) const = 0;
      //    Checks if ptr is already in the container.
      //    If yes then returns its position. 
      //    Returns -1 otherwise.

      virtual MIRContainerItem* operator[](int pos) const = 0;
      //    Returns the object at the given position

      virtual int findFirstElement(const MIRContainerKey* key) const = 0;
      //    Retrieves the first object in the container with the given key. 
      //    Returns the position of this object if successful 
      //    and -1 otherwise. 

      virtual int findLastElement(const MIRContainerKey* key) const = 0;
      //    Retrieves the last object in the container with the given key. 
      //    Returns the position of this object if successful 
      //    and -1 otherwise. 

      virtual int findNextElement(const MIRContainerKey* key, int pos) const = 0;
      //    Looks for an object with the given key at or after
      //    the specified position. Returns its index if successful 
      //    and NULL otherwise.

      virtual int findPreviousElement(const MIRContainerKey* key, int pos) const = 0;
      //    Looks for an object with the given key at or before 
      //    the specified position. Returns its index if successful 
      //    and NULL otherwise.

      virtual MIRContainerItem* removeElement(int pos) = 0;
      //    Removes the object contained at position pos. Note that 
      //    this operation may or may not preserve the order in 
      //    which the objects are stored.
      //    Returns a pointer to that object if successful and NULL 
      //    otherwise.
};

#endif /* end of file */
