//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2003    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################

#ifndef MIRBridgeSpecification_h
#define MIRBridgeSpecification_h

#include "MIRModelBridge.h"
#include "MIRString.h"
#include "MIRStringList.h"

/****************************************************
 *                                                  *
 * Class that implements the Bridge Specification   *
 *                                                  *
 ****************************************************/
class MIR_MODEL_BRIDGE_EXPORT MIRBridgeSpecification
{
public:
    //## Construction
    MIRBridgeSpecification() {};
    void initialize(const char* productWeb, const char* productCompany, const char* productName,
            const char* productVersion, const char* productMethodology, 
            char** productFileExtension, const char* bridgeVersion, const char*bridgeToolTip,
            bool inputIsDirectory=true, bool inputAllowEmptyPath=true);

    //## Destruction
    virtual ~MIRBridgeSpecification();

    //## DisplayName Attribute Operations
    inline virtual void setDisplayName(const MIRTString& value) {aDisplayName=value;};
    inline virtual const MIRString& getDisplayName() const {return aDisplayName;};
    //    Defines the Display Name attribute of the bridge.
    //    May be used to display different information

    //## ProductWeb Attribute Operations
    inline virtual void setProductWeb(const MIRTString& value) {aProductWeb=value;};
    inline virtual const MIRString& getProductWeb() const {return aProductWeb;};
    //    Defines the ProductWeb attribute of the bridge.

    //## ProductCompany Attribute Operations
    inline virtual void setProductCompany(const MIRTString& value) {aProductCompany=value;};
    inline virtual const MIRString& getProductCompany() const {return aProductCompany;};
    //    Defines the ProductCompany attribute of the bridge.

    //## ProductName Attribute Operations
    inline virtual void setProductName(const MIRTString& value) {aProductName=value;};
    inline virtual const MIRString& getProductName() const {return aProductName;};
    //    Defines the ProductName attribute of the bridge.


    //## ProductVersion Attribute Operations
    inline virtual void setProductVersion(const MIRTString& value) {aProductVersion=value;};
    inline virtual const MIRString& getProductVersion() const {return aProductVersion;};
    //    Defines the ProductVersion attribute of the bridge.


    //## ProductMethodology Attribute Operations
    inline virtual void setProductMethodology(const MIRTString& value) {aProductMethodology=value;};
    inline virtual const MIRString& getProductMethodology() const {return aProductMethodology;};
    //    Defines the ProductMethodology attribute of the bridge.

    //## BridgeVersion Attribute Operations
    inline virtual void setBridgeVersion(const MIRTString& value) {aBridgeVersion=value;};
    inline virtual const MIRString& getBridgeVersion() const {return aBridgeVersion;};
    //    Defines the BridgeVersion attribute of the bridge.

    //## ToolTip Attribute Operations
    inline virtual void setBridgeToolTip(const MIRTString& value) {aBridgeToolTip=value;};
    inline virtual const MIRString& getBridgeToolTip() const {return aBridgeToolTip;};
    //    Defines the BridgeToolTip attribute of the bridge.

    //## inputIsDirectory Attribute Operations
    inline virtual void setInputIsDirectory(MIRBoolean value) {aInputIsDirectory=value;};
    inline virtual const MIRBoolean getInputIsDirectory() const {return aInputIsDirectory;};
    //    Defines that the input path parameter is a directory

    //## inputAllowEmptyPath Attribute Operations
    inline virtual void setInputAllowEmptyPath(MIRBoolean value) {aInputAllowEmptyPath=value;};
    inline virtual const MIRBoolean getInputAllowEmptyPath() const {return aInputAllowEmptyPath;};
    //    Defines that the bridge allow empty path as a parameter

    //## ProductFileExtension Attribute Operations

    MIRBoolean addProductFileExtension(const MIRString& value);
    //    Associates the ProductFileExtension designated by the given value to the bridge.
    //    Returns TRUE if successful, or FALSE otherwise (if it exists).

    inline int getProductFileExtensionCount() const {return productFileExtensions.getSize();};
    //    Returns the number of ProductFileExtension(s) associated to the bridge.

    const MIRString& getProductFileExtension(int position) const;
    //    Returns the ProductFileExtension associated to the bridge
    //    located at the given position,
    //    Returns NULL if not successful.

    MIRBoolean removeProductFileExtension(int position);
    //    Removes the ProductFileExtension located at the given position.
    //    Returns TRUE if successful, or FALSE otherwise.

    void removeProductFileExtensions();
    //    Removes all ProductFileExtensions

protected:
    //## Attributes
    MIRString aDisplayName;
    MIRString aProductWeb;
    MIRString aProductCompany;
    MIRString aProductName;
    MIRString aProductVersion;
    MIRString aProductMethodology;
    MIRString aBridgeVersion;
    MIRString aBridgeToolTip;
    MIRBoolean aInputIsDirectory;
    MIRBoolean aInputAllowEmptyPath;

    MIRStringList productFileExtensions;
};

#endif

