//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2003    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################

#ifndef MIRCppInterface_h
#define MIRCppInterface_h

#include "MIRModelBridge.h"

/*
 * Function to test if a given import/export bridge is available for the caller
 */
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean isExportBridgeAvailable(const char* name);
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean isImportBridgeAvailable(const char* name);

/*
 * Function to retrieve the specification of an import/export bridge
 */
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean getExportBridgeSpecification(const char* name,MIRBridgeSpecification* specification);
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean getImportBridgeSpecification(const char* name,MIRBridgeSpecification* specification);

/*
 * Function to retrieve the list of options of an import/export bridge
 */
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean getExportBridgeOptions(const char* name,MIROptionList* options);
extern MIR_MODEL_BRIDGE_EXPORT MIRBoolean getImportBridgeOptions(const char* name,MIROptionList* options);

/*
 * Function to invoke a model conversion. The model conversion happens in a separate process.
 * This function is blocking: it will wait for the spanwed process to finish before returning.
 * The log of the conversion will be saved in the file specified by logFile.
 * The argument path allows to specify where to look for the executable:
 *      - if the value is NULL, will try to read from the registry
 *      - if the value is empty, will look in the directory of the parent process
 *      - if the value is non empty, use the absolute path provided
 */
extern MIR_MODEL_BRIDGE_EXPORT int MIRModelBridge(
        const char* importFile, const char* importTool, MIROptionList* importOptions, 
        const char* exportFile, const char* exportTool, MIROptionList* exportOptions,
        const char* logFile, short validLevel, const char* path=NULL);

/*
 * Function to invoke a model conversion. The model conversion happens in a separate process.
 * This function is blocking: it will wait for the spanwed process to finish before returning.
 * The log of the conversion will be stored in a MIRString.
 * The argument path allows to specify where to look for the executable:
 *      - if the value is NULL, will try to read from the registry
 *      - if the value is empty, will look in the directory of the parent process
 *      - if the value is non empty, use the absolute path provided
 */
extern MIR_MODEL_BRIDGE_EXPORT int MIRModelBridge(
        const char* importFile, const char* importTool, MIROptionList* importOptions, 
        const char* exportFile, const char* exportTool, MIROptionList* exportOptions,
        MIRLog* log, short validLevel, const char* path=NULL);

#endif
