//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2003    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################



#ifndef	MIRDate_h
#define	MIRDate_h 1

#include <time.h>

#include "MIRStdDefs.h"
#include "MIRString.h"

#if !defined(__SQLTYPES) && !defined(__SQLTYPES_H)
typedef struct tagDATE_STRUCT
{
    short            year;
    unsigned short   month;
    unsigned short   day;
} DATE_STRUCT;

typedef struct tagTIME_STRUCT
{
    unsigned short   hour;
    unsigned short   minute;
    unsigned short   second;
} TIME_STRUCT;

typedef struct tagTIMESTAMP_STRUCT
{
    short            year;
    unsigned short   month;
    unsigned short   day;
    unsigned short   hour;
    unsigned short   minute;
    unsigned short   second;
    unsigned int   fraction;
} TIMESTAMP_STRUCT;
#endif

/*
 * This class provides a uniform way to represent dates, time and timestamps.
 */
class MIR_STD_API MIRDate
{
  public:

    //## Construction

      MIRDate();
      MIRDate(DATE_STRUCT arg);
      MIRDate(TIME_STRUCT arg);
      MIRDate(TIMESTAMP_STRUCT arg);
      MIRDate(time_t arg);

      static void* operator new(size_t size);
      static void operator delete(void* ptr);
#ifdef WIN32
      static void* operator new(size_t size, const char* file, int line);
      static void* operator new(size_t size, int block_type, const char* file, int line);
#if _MSC_VER >= 1200
      static void operator delete(void* ptr, const char* file, int line);
      static void operator delete(void* ptr, int block_type, const char* file, int line);
#endif
#endif
      //    Used to allocate/deallocate MIRDate objects (and subclasses). 


    //## Comparison

      bool operator==(const MIRDate& s) const;
      bool operator!=(const MIRDate& s) const;
      bool operator<(const MIRDate& s) const;
      bool operator>(const MIRDate& s) const;
      bool operator<=(const MIRDate& s) const;
      bool operator>=(const MIRDate& s) const;
      MIRDate operator+(const MIRDate& s1) const;
      MIRDate operator-(const MIRDate& s1) const;
      void operator=(const MIRDate&);
      void operator=(const TIME_STRUCT *);
      void operator=(const DATE_STRUCT *);
      void operator=(const TIMESTAMP_STRUCT *);
      //    These operators compare two MIRDates and return 
      //    true or false as appropriate.


    //## Concatenation

      MIRDate& add(const MIRDate* first, ...);

      void setDate();

      void getDate(DATE_STRUCT *) const;

      void getDate(TIME_STRUCT *) const;

      void getDate(TIMESTAMP_STRUCT *) const;

      DATE_STRUCT* getDate() const;

      TIME_STRUCT* getTime() const;

      void setDate(const TIME_STRUCT *);

      void setDate(const DATE_STRUCT *);

      void setDate(const TIMESTAMP_STRUCT *);

      const char* getHour() const;

      bool isEmpty() const; 

      void setDate(MIRString& newDate);
      //    Allows the date to be set from a time struct type of string
      //    It would have the form of yyyymmddhhmmss

      void setDate(time_t newDate);
      operator time_t() const;

    //## General methods

      void empty();
      //    Clears the date.

      operator const char*() const;
      const char* chars() const;
      //    Return a pointer to the '\0' terminated character buffer.

	  const char* getODBCDate() const;
	  const char* getODBCTimestamp() const;
	  //    Return the date or date/time in the ODBC escape format
	  //    Use getHour() to get the time

  protected:
      TIMESTAMP_STRUCT theDate;
      char dateBuffer[30];
      bool isNull;

      int compare(const MIRDate& sDate) const;
};


/*
 * MIRDate Inline methods
 */

#ifdef WIN32
inline void* MIRDate::operator new(size_t size, const char* file, int line)
{
    return operator new(size,_NORMAL_BLOCK,file,line);
}

#if _MSC_VER >= 1200
inline void MIRDate::operator delete(void* ptr, int block_type, const char* file, int line)
{
    operator delete(ptr);
}

inline void MIRDate::operator delete(void* ptr, const char* file, int line)
{
    operator delete(ptr);
}
#endif
#endif


//## MIRDate Comparison


inline int MIRDate::compare(const MIRDate& s) const
{
    // Check isNull first because,
    // if isNull is set, theDate may not be set
    if (isNull && s.isNull)
        return 0;
    else if (isNull)
        return -1;
    else if (s.isNull)
        return 1;

    // theDate is set on both dates
    int res;
    res=theDate.year - s.theDate.year;
    if (res==0) {
        res=theDate.month - s.theDate.month;
        if (res==0) {
            res=theDate.day - s.theDate.day;
            if (res==0) {
                res=theDate.hour - s.theDate.hour;
                if (res==0) {
                    res=theDate.minute - s.theDate.minute;
                    if (res==0) {
                        res=theDate.second - s.theDate.second;
                        if (res==0) {
                            res=theDate.fraction - s.theDate.fraction;
                        }
                    }
                }
            }
        }
    }
    return res;
}

inline bool MIRDate::operator==(const MIRDate& s) const
{
    return compare(s)==0;
}

inline bool MIRDate::operator!=(const MIRDate& s) const
{
    return compare(s)!=0;
}

inline bool MIRDate::operator<(const MIRDate& s) const
{
    return compare(s)<0;
}

inline bool MIRDate::operator>(const MIRDate& s) const
{
    return compare(s)>0;
}

inline bool MIRDate::operator<=(const MIRDate& s) const
{
    return compare(s)<=0;
}

inline bool MIRDate::operator>=(const MIRDate& s) const
{
    return compare(s)>=0;
}

inline bool MIRDate::isEmpty() const
{
    return isNull;
}

#endif /* end of file */
