//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2002    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################



#ifndef MIRExceptionObject_h
#define MIRExceptionObject_h 1


#include "MIRString.h"

/*
 * This object is used to carry MIR exceptions.
 */
class MIR_STD_API MIRException 
{
public:
    static void* operator new(size_t size);
    static void operator delete(void* ptr);
#ifdef WIN32
    static void* operator new(size_t size, const char* file, int line);
    static void* operator new(size_t size, int block_type, const char* file, int line);
#if _MSC_VER >= 1200
    static void operator delete(void* ptr, const char* file, int line);
    static void operator delete(void* ptr, int block_type, const char* file, int line);
#endif
#endif
    //    Used to allocate/deallocate MIRException objects (and subclasses). 

private:
    //## Construction

    MIRException(int number, const MIRString& message);
    //    Initializes this MIRException using the provided number
    //    and message.

public:
    friend class MIRLog;

    MIRException(const MIRString& message);

    MIRException(const MIRException& right);
    //    Initialises this MIRException using the provided template

    //## Destruction

    virtual ~MIRException();
    //    Releases the resources used by this object.

    virtual const MIRString& getErrorMessage() const;
    virtual const int getErrorNumber() const;

protected:
    int errorNumber;
    MIRString errorMessage;
};



/*
 * Inline methods
 */

#ifdef WIN32
inline void* MIRException::operator new(size_t size, const char* file, int line)
{
    return operator new(size,_NORMAL_BLOCK,file,line);
}

#if _MSC_VER >= 1200
inline void MIRException::operator delete(void* ptr, int block_type, const char* file, int line)
{
    operator delete(ptr);
}

inline void MIRException::operator delete(void* ptr, const char* file, int line)
{
    operator delete(ptr);
}
#endif
#endif


inline const int MIRException::getErrorNumber() const
{
    return errorNumber;
}

inline const MIRString& MIRException::getErrorMessage() const
{
    return errorMessage;
}


#endif /* end of file */
