//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2003    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################

#ifndef MIRModelBridge_h
#define MIRModelBridge_h

#if defined(WIN32)
#    ifdef __IN_MIR_MODEL_BRIDGE__
#        define MIR_MODEL_BRIDGE_EXPORT __declspec(dllexport)
#    else
#        define MIR_MODEL_BRIDGE_EXPORT __declspec(dllimport)
#    endif /* __IN_MIR_MODEL_BRIDGE__ */
#else
#    define MIR_MODEL_BRIDGE_EXPORT
#endif /* WIN32 */

#define MIR_MODEL_BRIDGE_CAPI "C" MIR_MODEL_BRIDGE_EXPORT

class MIRBridgeSpecification;
class MIRLog;

#include "MIRBridgeSpecification.h"
#include "MIROption.h"
#include "MIRStdDefs.h"
#include "MIRString.h"
#include "MIRStringList.h"
#include "MIRConfiguration.h"

#ifdef WIN32
#include <windows.h>
typedef HINSTANCE DLLINSTANCE;
#else
#include <dlfcn.h>
typedef void* DLLINSTANCE;
#endif

// Some callback types
typedef void (* DLLOPTIONFUNC)(MIROptionList *);
typedef void (* DLLSPECFUNC)(MIRBridgeSpecification *);

extern MIR_MODEL_BRIDGE_CAPI void MIRModelBridgeMessages(MIRString** messages, int* offset, int* size);
extern MIR_MODEL_BRIDGE_CAPI char* MIRModelBridgeI18nXmlFile();

#define MODEL_BRIDGE_OFFSET 400

enum errorMessages {
    MODEL_BRIDGE_NO_ERROR=MODEL_BRIDGE_OFFSET,
    MODEL_BRIDGE_CANNOT_LOAD_DLL,
    MODEL_BRIDGE_CANNOT_FIND_ENTRY_POINT,
    MODEL_BRIDGE_CANNOT_OPEN_CONFIGURATION_FILE,
    MODEL_BRIDGE_CONFIGURATION_FILE_INVALID
};

/************************************************
 *                                              *
 * Class that implements the Dll Info Structure *
 *                                              *
 ************************************************/

class MIR_MODEL_BRIDGE_EXPORT DllInfo
{
    friend class MIRBridgesManager;
public:
    //## Construction
    DllInfo(const char* _name): name(_name), hDLL(NULL),getSpecification(NULL),getOptions(NULL) {};

    //## Destruction
    virtual ~DllInfo();

    inline const char* getName() {return name;};
    inline int getDisplayNameCount() {return displayNames.getSize();};
    const char* getDisplayName(int position);
    void addDisplayName(const char* displayName);
protected:
    MIRString       name;
    MIRStringList   displayNames;
    DLLINSTANCE     hDLL;
    DLLSPECFUNC     getSpecification;   // Specification Function pointer
    DLLOPTIONFUNC   getOptions;         // Bridge Options Function pointer
};

/************************************************
 *                                              *
 * Class that implements the Bridges Manager    *
 *                                              *
 ************************************************/

enum managerInitialization {
    MANAGER_INIT_NONE=0,
    MANAGER_INIT_ARCH,
    MANAGER_INIT_INI,
    MANAGER_INIT_XML
};

class MIR_MODEL_BRIDGE_EXPORT MIRBridgesManager
{
public:
    //## Construction
    MIRBridgesManager(MIRBoolean isExport,int initMethod);
    //    Obtain the list of available import or export bridges.

    //## Destruction
    virtual ~MIRBridgesManager();
    //    Release the resources used by this object

    //## Utility methods
    MIRBoolean initialize(int initMethod);
    //    Reinitiliaze the Manager: reset the error and the list of available bridges.
    //    Obtain a new list of available import or export bridges.

    inline int size() {return dllInfos.getSize();};
    //    Return the number of DLLs currently available

    DllInfo* getDllInfo(int position);
    //    Return the DllInfo structure at the given position in the dllInfos container.
    //    Return NULL if position is out of bound.

    DllInfo* getDllInfo(const char* name);

    MIRBoolean isAvailable(const char* name);
    //    Return a pointer on the dll info structure if successful, NULL otherwise.

    virtual MIRBoolean getBridgeSpecification(const char* name,MIRBridgeSpecification* specification);
    //    Call the specification function and fill the given structure.
    //    Return true if successful, false otherwise.

    virtual MIRBoolean getBridgeOptions(const char* name,MIROptionList* options);
    //    Call the bridge options function and fill the given srtucture.
    //    Return true if successful, false otherwise.

    inline int getLastError() {return lastError;};
    //    Return the code of the last error that occured..

    const char* getLastErrorMessage();
    //    Return an internationalized message describing the last error that occured.

protected:
    friend class MIRInitFromXml;

    void cleanup();
    virtual DllInfo* newDllInfo(const char* name);
    DllInfo* loadBridge(const char* name);

    int lastError;
    MIRVector dllInfos;
    MIRBoolean isExport;
    MIRString operation;
    MIRBoolean compatible;
};

#endif
