//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2004    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################


#ifndef MIRPaths_h
#define MIRPaths 1

#include <ctype.h>

#include "MIRStdDefs.h"
#include "MIRString.h"

#ifdef WIN32
#define INLINE extern MIR_STD_CAPI
#else
#include <unistd.h>
/* The Sun compiler does not want a function to be declared extern and then inline */
#define INLINE inline
#endif

extern MIR_STD_API MIRTString getTmpFileName(const char* dirName=NULL);

INLINE MIRBoolean isAPathSlash(char c);
//    Returns true if the character is a directory separator like '/'.

INLINE MIRBoolean isAPathColon(char c);
//    Returns true if the character is a path separator like ':' on Unix.

INLINE MIRBoolean isAbsolute(const char* path);
INLINE MIRBoolean isRelative(const char* path);
//    isAbsolute returns true if the given path is absolute.
//    isRelative returns true if the given path is relative.
//    Note that some paths are neither absolute nor relative.
//    Examples:
//      Path        isAbsolute    isRelative
//      file.c        false         true
//      dir\file.c    false         true
//      c:file.c      false         false
//      \file.c       false         false
//      c:\file.c     true          false

INLINE const char* findPathSlash(const char* path);
//    Returns a pointer to the first directory separator in the given path.

INLINE const char* reverseFindPathSlash(const char* path);
//    Returns a pointer to the last directory separator in the given path.

INLINE const char* findPathColon(const char* path);
//    Returns a pointer to the first path separator in the given path list.

INLINE const char* reverseFindPathColon(const char* path);
//    Returns a pointer to the last path separator in the given path list.

// CHR_PATH_SLASH
// STR_PATH_SLASH
//    These macro define the platform's natural directory separator, i.e.
//    '\\' on Windows and '/' on Unix. CHR is a character '/' and STR is a 
//    string "/".

// CHR_PATH_COLON
// STR_PATH_COLON
//    These macro define the platform's natural path separator, i.e.
//    ';' on Windows and ':' on Unix. CHR is a character ':' and STR is a 
//    string ":".


/*
 * Inline methods
 */

#ifdef WIN32

#define CHR_PATH_SLASH  '\\'
#define STR_PATH_SLASH  "\\"

#define CHR_PATH_COLON  ';'
#define STR_PATH_COLON  ";"

inline MIRBoolean isAPathSlash(char c)
{
    return ((c=='/') || (c=='\\'));
}

#else /* !WIN32 */

#define CHR_PATH_SLASH  '/'
#define STR_PATH_SLASH  "/"

#define CHR_PATH_COLON  ':'
#define STR_PATH_COLON  ":"

inline MIRBoolean isAPathSlash(char c)
{
    return (c==CHR_PATH_SLASH);
}

inline MIRBoolean isAbsolute(const char* path)
{
    return path[0]==CHR_PATH_SLASH;
}

inline MIRBoolean isRelative(const char* path)
{
    return path[0]!=CHR_PATH_SLASH;
}

#endif /* WIN32 */

inline MIRBoolean isAPathColon(char c)
{
    return (c==CHR_PATH_COLON);
}

inline const char* findPathSlash(const char* path)
{
    while (*path) {
        if (isAPathSlash(*path))
            return path;
        path++;
    }
    return NULL;
}

inline const char* reverseFindPathSlash(const char* path)
{
    const char* last=NULL;
    while (*path) {
        if (isAPathSlash(*path))
            last=path;
        path++;
    }
    return last;
}

inline const char* findPathColon(const char* path)
{
    while (*path) {
        if (isAPathColon(*path))
            return path;
        path++;
    }
    return NULL;
}

inline const char* reverseFindPathColon(const char* path)
{
    const char* last=NULL;
    while (*path) {
        if (isAPathColon(*path))
            last=path;
        path++;
    }
    return last;
}

#endif /* end of file */
