//####################################################################
//##     Copyright  Meta Integration Technology, Inc. 1997-2002    ##
//##                      All Rights Reserved.                      ##
//####################################################################
//## Meta Integration(tm) Model Bridge (MIMB), Works (MIW),         ##
//## Repository (MIR), Repository SDK (MIRSDK), and Database SDK    ##
//## (MIDBSDK) are trademarks of Meta Integration Technology, Inc.  ##
//## Other product and company names (or logos) mentioned herein    ##
//## may be the trademarks of their respective owners.              ##
//####################################################################
//##                 http://www.metaintegration.com                 ##
//####################################################################



#ifndef MIRStringList_h
#define MIRStringList_h 1


#include "MIRString.h"
#include "MIRVector.h"

/*
 * This class provides and ordered container for String lists.
 */
class MIR_STD_API MIRStringList: public MIROrderedVector
{

  public:

    //## Construction

      MIRStringList();
      //    Creates an empty string list.

      MIRStringList(int init_size);
      //    Creates a string list that will allocate memory for 
      //    init_size elements when the first element is added.

      MIRStringList(const MIRStringList& src);
      //    Creates a new string list initialising it with the contents of 
      //    the given source string list. Note that only the object 
      //    pointers are copied, the objects themselves are not duplicated.

      typedef MIROrderedVector::Iterator Iterator;

  protected:

    //## Ordering functions
      // Objects put in containers are supposed to have a key. 
      // The following compare functions define an order on these keys.
      // This allows searches to be performed and the implementation of 
      // containers that require that their elements be sorted.

      virtual int compare2Object(const MIRContainerItem* ptr1, const MIRContainerItem* ptr2) const;
      //    Compares the keys of two objects. This operation is used 
      //    when inserting/removing an object and in the contains function.
      //    Returns <0 if key of ptr1 < key of ptr2
      //             0 if key of ptr1 == key of ptr2
      //            >0 if key of ptr1 > key of ptr2
      //    By default this function compares the pointer values of 
      //    ptr1 and ptr2.

      virtual int compare2Key(const MIRContainerItem* ptr, const MIRContainerKey* key) const;
      //    Compares the key of ptr with the given key. This operation is used
      //    when in the functions that look for objects corresponding to a
      //    specific key. It is not used when inserting/removing an object or 
      //    in the contains function.
      //    Returns <0 if key of ptr1 < key
      //             0 if key of ptr1 == key
      //            >0 if key of ptr1 > key
      //    By default this function compares the pointer value of ptr
      //    with that of key.
};

typedef MIRStringList::Iterator MIRStringListIterator;

/*
 * Inline operations
 */
inline MIRStringList::MIRStringList():
    MIROrderedVector()
{
}

inline MIRStringList::MIRStringList(int init_size):
    MIROrderedVector(init_size)
{
}

inline MIRStringList::MIRStringList(const MIRStringList& src):
    MIROrderedVector(src)
{
}

inline int MIRStringList::compare2Object(const MIRContainerItem* ptr1, const MIRContainerItem* ptr2) const
{
    return ((MIRString*)ptr1)->compare(*((MIRString*)ptr2));
}

inline int MIRStringList::compare2Key(const MIRContainerItem* ptr, const MIRContainerKey* key) const
{
    return ((MIRString*)ptr)->compare(*(MIRString*)key);
}

#endif /* end of file */
